<?php
namespace Teknisa\Model\Gestaopessoas\Repositories;

use HCM\Util\DateUtil;
use HCM\Util\Repositories;
use Util\Util;

class EsoLote extends \Doctrine\ORM\EntityRepository {

    public function enfileiraEnvio($nrorg, $nrlote, $nrconsumer, $nrenvio, $nmevento, $cdoperador) {
        try {
            $this->getEntityManager()->beginTransaction();
            
            $sql = $this->getEntityManager()->getConnection()->prepare(
                "SELECT L.* FROM ESO_LOTE L WHERE NRORG = :P_NRORG AND NRCONSUMER = :P_NRCONSUMER AND NRENVIO = :P_NRENVIO AND IDSITUACAO IN ('PROCESSAMENTO')"
            );
    
            $sql->execute(
                array(
                    'P_NRORG' => $nrorg,
                    'P_NRCONSUMER' => $nrconsumer,
                    'P_NRENVIO' => $nrenvio,
                )
            );

            $lote = $sql->fetch();

            if (!$lote) {
                $sql = $this->getEntityManager()->getConnection()->prepare("
        UPDATE ESO_PERIODICO
           SET NRLOTE = :NRLOTE, DTULTATU = :DTULTATU
         WHERE NRORG = :NRORG AND NRENVIO = :NRENVIO");
                $sql->execute(
                    array(
                        "NRORG" => $nrorg,
                        "NRENVIO" => $nrenvio,
                        "NRLOTE" => $nrlote,
                        "DTULTATU" => DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA),
                    )
                );
    
                $sql = $this->getEntityManager()->getConnection()->prepare("
        INSERT INTO ESO_LOTE
            (NRORG, NRLOTE, NRCONSUMER, NRENVIO, NMEVENTO, DTINCLUSAO, CDOPERINCLUSAO, NRORGINCLUSAO)
        VALUES 
            (:NRORG, :NRLOTE, :NRCONSUMER, :NRENVIO, :NMEVENTO, :DTINCLUSAO, :CDOPERINCLUSAO, :NRORGINCLUSAO)");
                $sql->execute(
                    array(
                        "NRORG" => $nrorg,
                        "NRLOTE" => $nrlote,
                        "NRCONSUMER" => $nrconsumer,
                        "NRENVIO" => $nrenvio,
                        "NMEVENTO" => $nmevento,
                        "DTINCLUSAO" => DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA),
                        "CDOPERINCLUSAO" => $cdoperador,
                        "NRORGINCLUSAO" => $nrorg,
                    )
                );
            }

            $this->getEntityManager()->commit();
        } catch(\Exception $e) {
            $this->getEntityManager()->rollback();
            Util::writeLogFile($nrorg, "error enfileiraEnvio (Lote:$nrlote Envio: $nrenvio) [Code:".$e->getCode()." Message:".$e->getMessage()."]","consumer$nrconsumer");
            throw $e;
        }
    }

    public function getProximoLoteFila($nrorg, $nrconsumer)
    {
        $sql = $this->getEntityManager()->getConnection()->prepare(
            "SELECT L.* FROM ESO_LOTE L WHERE NRORG = :P_NRORG AND NRCONSUMER = :P_NRCONSUMER AND IDSITUACAO = 'PENDENTE' ORDER BY DTINCLUSAO"
        );

        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRCONSUMER' => $nrconsumer
            )
        );

        return $sql->fetch();
    }

    public function getEventosLote($nrorg, $nrconsumer, $nrlote)
    {
        $sql = $this->getEntityManager()->getConnection()->prepare(
            "SELECT L.* FROM ESO_LOTE L WHERE NRORG = :P_NRORG AND NRCONSUMER = :P_NRCONSUMER AND NRLOTE = :P_NRLOTE"
        );

        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRCONSUMER' => $nrconsumer,
                'P_NRLOTE' => $nrlote
            )
        );

        return $sql->fetchAll();
    }

    public function atualizaStatusLoteProcessamento($nrorg, $cdoperador, $nrconsumer, $nrlote, $nrenvio = null)
    {
        return $this->atualizaStatusLote($nrorg, $cdoperador, $nrconsumer, $nrlote, $nrenvio, 'PROCESSAMENTO');
    }

    public function atualizaStatusLoteConcluido($nrorg, $cdoperador, $nrconsumer, $nrlote, $nrenvio = null)
    {
        return $this->atualizaStatusLote($nrorg, $cdoperador, $nrconsumer, $nrlote, $nrenvio, 'CONCLUIDO');
    }

    public function atualizaStatusLoteErro($nrorg, $cdoperador, $nrconsumer, $nrlote, $nrenvio = null)
    {
        return $this->atualizaStatusLote($nrorg, $cdoperador, $nrconsumer, $nrlote, $nrenvio, 'ERRO');
    }

    public function atualizaStatusLote($nrorg, $cdoperador, $nrconsumer, $nrlote, $nrenvio, $idsituacao)
    {
        $sql = $this->getEntityManager()->getConnection()->prepare(
            "UPDATE ESO_LOTE
                SET IDSITUACAO = :P_IDSITUACAO, DTULTATU = :P_DTULTATU, NRORGULTATU = :P_NRORG, CDOPERULTATU = :P_CDOPERADOR
              WHERE NRORG = :P_NRORG
                AND NRCONSUMER = :P_NRCONSUMER
                AND NRLOTE = :P_NRLOTE
                AND ( NRENVIO = :P_NRENVIO OR :P_NRENVIO IS NULL )"
        );

        $sql->execute(
            array(
                'P_NRORG' => $nrorg,
                'P_NRCONSUMER' => $nrconsumer,
                'P_NRLOTE' => $nrlote,
                'P_NRENVIO' => $nrenvio,
                'P_CDOPERADOR' => $cdoperador,
                'P_IDSITUACAO' => $idsituacao,
                'P_DTULTATU' => DateUtil::getDataAtual()->format(DateUtil::FORMATO_BRASILEIRO_DATAHORA),
            )
        );
    }
}
